@@ -127,6 +127,25 @@ progress_set_format (GladeXML *xml, GtkWidget *w,
     gtk_progress_set_format_string (GTK_PROGRESS (w), value);
 }
 
+static void
+list_item_set_label (GladeXML *xml, GtkWidget *w,
+		     const char *name, const char *value)
+{
+    GtkWidget *child;
+
+    child = gtk_bin_get_child (GTK_BIN (w));
+    if (child)
+	gtk_container_remove (GTK_CONTAINER (w), child);
+    
+    /* copied from gtk_list_item_new_with_label() */
+    child = gtk_label_new (value);
+    gtk_misc_set_alignment (GTK_MISC (child), 0.0, 0.5);
+    gtk_misc_set_padding (GTK_MISC (child), 0, 1);
+
+    gtk_container_add (GTK_CONTAINER (w), child);
+    gtk_widget_show (child);
+}
+
 static void
 text_view_set_text (GladeXML *xml, GtkWidget *w,
 		    const char *name, const char *value)
@@ -384,6 +403,14 @@ window_set_wmclass_class (GladeXML *xml, GtkWidget *w,
     g_free(wmname);
 }
 
+static void
+button_set_response_id (GladeXML *xml, GtkWidget *w,
+			const gchar *name, const gchar *value)
+{
+    g_object_set_data (G_OBJECT (w), "response_id",
+		       GINT_TO_POINTER (INT (value)));
+}
+
 static void
 menuitem_build_children(GladeXML *self, GtkWidget *w,
 			GladeWidgetInfo *info)
@@ -423,19 +450,24 @@ gtk_dialog_build_children(GladeXML *self, GtkWidget *w,
     if (dialog->action_area == NULL)
 	return;
 
-    children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
+    /* repack children of action_area */
+    children = gtk_container_get_children(GTK_CONTAINER(dialog->action_area));
     for (list = children; list; list = list->next) {
-	gtk_widget_ref (GTK_WIDGET (list->data));
-	gtk_container_remove (GTK_CONTAINER (dialog->action_area), GTK_WIDGET (list->data));
+	GtkWidget *child = GTK_WIDGET(list->data);
+
+	g_object_ref(child);
+	gtk_container_remove (GTK_CONTAINER (dialog->action_area), child);
     }
+    for (list = children; list; list = list->next) {
+	GtkWidget *child = GTK_WIDGET(list->data);
+	gint response_id;
+
+	response_id = GPOINTER_TO_INT(g_object_steal_data(G_OBJECT(child),
+							  "response_id"));
+	gtk_dialog_add_action_widget(dialog, child, response_id);
+	g_object_unref(child);
 
-    for (list = children; list; list = list->next) {
-	gint response_id;
-	response_id = GPOINTER_TO_INT (g_object_steal_data (
-	    G_OBJECT (list->data), "response_id"));
-	gtk_dialog_add_action_widget (dialog, GTK_WIDGET (list->data), response_id);
     }
-    g_list_foreach (children, (GFunc)gtk_widget_unref, NULL);
     g_list_free (children);
 }
 
@@ -512,7 +544,19 @@ option_menu_build_children (GladeXML *xml, GtkWidget *parent,
 {
     int i, history = 0;
 
-    glade_standard_build_children (xml, parent, info);
+    for (i = 0; i < info->n_children; i++) {
+	GtkWidget *child;
+	GladeWidgetInfo *childinfo = info->children[i].child;
+
+	if (strcmp(childinfo->class, "GtkMenu") != 0) {
+	    g_warning("the child of the option menu '%s' was not a GtkMenu",
+		      info->name);
+	    continue;
+	}
+	child = glade_xml_build_widget(xml, childinfo);
+
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(parent), child);
+    }
 
     for (i = 0; i < info->n_properties; i++) {
 	const char *name  = info->properties[i].name;
@@ -789,29 +833,6 @@ layout_build_children (GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
     }
 }
 
-static GtkWidget *
-build_button(GladeXML *xml, GType widget_type,
-	     GladeWidgetInfo *info)
-{
-    GtkWidget *widget;
-    gint i, response_id = 0;
-
-    for (i = 0; i < info->n_properties; i++) {
-	if (!strcmp (info->properties[i].name, "response_id")) {
-	    response_id = INT (info->properties[i].value);
-	    break;
-	}
-    }
-
-    widget = glade_standard_build_widget (xml, widget_type, info);
-
-    if (response_id)
-	g_object_set_data (G_OBJECT (widget), "response_id",
-			   GINT_TO_POINTER (response_id));
-
-    return widget;
-}
-
 static GtkWidget *
 dialog_find_internal_child(GladeXML *xml, GtkWidget *parent,
 			   const gchar *childname)
@@ -844,26 +865,6 @@ image_menu_find_internal_child(GladeXML *xml, GtkWidget *parent,
     return NULL;
 }
 
-static GtkWidget *
-option_menu_find_internal_child(GladeXML *xml, GtkWidget *parent,
-				const gchar *childname)
-{
-    if (!strcmp(childname, "menu")) {
-	GtkWidget *ret;
-	
-	if ((ret = gtk_option_menu_get_menu (GTK_OPTION_MENU (parent))))
-	    return ret;
-
-	ret = gtk_menu_new ();
-	gtk_widget_show (ret);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (parent), ret);
-
-	return ret;
-    }
-
-    return NULL;
-}
-
 static GtkWidget *
 scrolled_window_find_internal_child(GladeXML *xml, GtkWidget *parent,
 				    const gchar *childname)
@@ -982,6 +983,8 @@ _glade_init_gtk_widgets(void)
     glade_register_custom_prop (GTK_TYPE_MENU_ITEM, "use_stock", menu_item_set_use_stock);
     glade_register_custom_prop (GTK_TYPE_WINDOW, "wmclass_name", window_set_wmclass_name);
     glade_register_custom_prop (GTK_TYPE_WINDOW, "wmclass_class", window_set_wmclass_class);
+    glade_register_custom_prop (GTK_TYPE_LIST_ITEM, "label", list_item_set_label);
+    glade_register_custom_prop (GTK_TYPE_BUTTON, "response_id", button_set_response_id);
 
     glade_register_widget (GTK_TYPE_ACCEL_LABEL, glade_standard_build_widget,
 			   NULL, NULL);
@@ -991,7 +994,7 @@ _glade_init_gtk_widgets(void)
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_ASPECT_FRAME, glade_standard_build_widget,
 			   glade_standard_build_children, NULL);
-    glade_register_widget (GTK_TYPE_BUTTON, build_button,
+    glade_register_widget (GTK_TYPE_BUTTON, glade_standard_build_widget,
 			   glade_standard_build_children, NULL);
     glade_register_widget (GTK_TYPE_CALENDAR, glade_standard_build_widget,
 			   NULL, NULL);
@@ -1072,7 +1075,7 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_NOTEBOOK, glade_standard_build_widget,
 			   notebook_build_children, NULL);
     glade_register_widget (GTK_TYPE_OPTION_MENU, glade_standard_build_widget,
-			   option_menu_build_children, option_menu_find_internal_child);
+			   option_menu_build_children, NULL);
     glade_register_widget (GTK_TYPE_PIXMAP, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_PLUG, NULL,
@@ -1090,6 +1093,8 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_SCROLLED_WINDOW, glade_standard_build_widget,
 			   glade_standard_build_children,
 			   scrolled_window_find_internal_child);
+    glade_register_widget (GTK_TYPE_SEPARATOR_MENU_ITEM, glade_standard_build_widget,
+			   NULL, NULL);
     glade_register_widget (GTK_TYPE_SOCKET, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_SPIN_BUTTON, glade_standard_build_widget,
